import { useCallback, useEffect, useState } from "react";
import { effectUtils } from './utils';
import { Subscription } from 'rxjs';
import { getFirebaseReadsLogger } from "./utils/getFirebaseReadsLogger";

export const usePageReadsLogger = () => {
  const [pageReads, setPageReads] = useState(0);
  const [lastPageReads, setLastPageReads] = useState(0);


  useEffect(() => {
    let pageSubscription: Subscription = null,
      lastPageSubscription: Subscription = null;

    effectUtils.init(
      'page',
      setPageReads,
      setLastPageReads
    ).then(([pageSub, lastPageSub]) => {
      pageSubscription = pageSub;
      lastPageSubscription = lastPageSub;
    });

    return () => effectUtils.clean(
      pageSubscription,
      lastPageSubscription
    );
  }, [setPageReads, setLastPageReads]);

  const resetPageReads = useCallback(async () => {
    const logger = await getFirebaseReadsLogger();
    if (logger) {
      logger.resetPageReads();
    }
  }, []);

  return [pageReads, lastPageReads, resetPageReads];
};
